package Puma::Config;
use ibase Exporter;

=head1 NAME

Puma::Config - finds and loads the Puma config.xml file

=head1 SYNOPSIS

 use Puma::Config qw( getConfig );
 my $config = getconfig('/etc/puma/config.xml');

=head1 DESCRIPTION

The Puma::Core package provides a unified construction mechanism with full
inheritance support for all subclasses.

Subclasses should not define a new() constructor but should instead rely on
the construct() mechanism as defined below.

=head1 FUNCTIONS

=head2 getConfig([FILENAME]);

Returns a hash of values extracted for the config.xml file formatted via the
Puma::Util::XML module.

If FILENAME is not defined, it will look first in /etc/puma and then in
/usr/local/etc/puma for the config.xml file.

If no file is found, an empty hash is returned.

=cut

@EXPORT_OK = qw( getConfig );

use Puma::Util::XML 'xml2hash';

use strict;

sub getConfig {
	my ($class, $filename) = @_;

	unless ($filename) {
		if (-e '/etc/puma/config.xml') {
			$filename = '/etc/puma/config.xml';
		} elsif (-e '/usr/local/etc/puma/config.xml') {
			$filename = '/usr/local/etc/puma/config.xml';
		} else {
			return {};
		}
	}

	return xml2hash(FileName => $filename) || {};
}

=head1 COPYRIGHT

Copyright (c) 2001, 2002 by Ingmar Ellenberger.

Distributed under The Artistic License.  For the text of this license,
see http://puma.site42.com/license.psp or read the file LICENSE in the
root of the distribution.

=cut

1;
