package Puma::ServerPages::Include;
use ibase Exporter;

@EXPORT_OK = qw( include );

use strict;

# === Server-side Include ===================================================

=method

	include - Compile time server-side include

	Added as a part of the new Handler mechanism for the parser, this method
   allows server-side includes whenever the ':include' label is found.

=cut

use Puma::Util qw( fixpath quote splithash );
use Puma::Util::NCHash;
sub include {
	shift if $_[0] eq 'Puma::ServerPages::Include';
	
	# --- grab self as parser if being used as a method call ---
	#! TODO: ?fix? this dysfunctional piece of code
	my $parser;
	#$parser = shift
	#	if $parser && ref $parser && $parser =~ /=/ && $parser->isa('Puma::Parser');

	# --- parameter processing ---
	my $args = nchash(@_);
	my $params = splithash($args, qw(
		file server config parser
		fspath webpath inline
	));

#use Data::Dumper;
#print Data::Dumper->Dump([$args], ['args']) ."\n";
#print Data::Dumper->Dump([$params], ['params']) ."\n";

	# --- file="..." required ---
	return "ServerPages::include - no filename" unless $params->{file};

	# --- load configuration data ---
	my $config = $params->{server} || $params->{config};
	$config = $config->getConfig 
		if $config && ref($config) && $config =~ /=/ && $config->can('getConfig');

	# --- find the root path ---
	#! TODO: Use $ENV{INCLUDE_ROOT}
	#!       Depricate the entire $config parameter?
	my $path = $config->{include}->{root} || $ENV{DOCUMENT_ROOT} || '/'; # default path

	$path = '/'                    if defined $params->{fspath};
	$path = $ENV{DOCUMENT_ROOT}    if defined $params->{webpath};
	#$path = $config->{include}->{root} if defined $params->{include}->{root};

	# --- resolve filename ---
	my $file = fixpath($path, $params->{file});

#print "path = $path, $params->{file}, $file\n";

	# --- create the parser ---
	unless ($parser) { # do we already have a parser (self)?
		if ($params->{parser}) {
			$parser = $params->{parser}->new;
		} else {
			$parser = new Puma::ServerPages::Parser;
		}
	}

	# --- load the file and parse it ---
	my $ptree = $parser->parseFile($file);

	# --- remove the :parser tag and its elements ---
	splice @{$ptree}, 0, 2;

	# --- process unused params as code $include->{key} = value ---
	my $code = "   my \$include;\n";
	foreach my $key (keys %$args) {
		my $value = $args->{$key};

		# --- quote the value unless it's an eval (backquotes) ---
		if ($value =~ /^`([^`]+)`$/) {
			$value = $1;
		} else {
			$value = quote($value);
		}

		$code .= "	\$include->{$key} = $value;\n";
	}
	unshift @{$ptree}, [ ':', { code => $code } ] if $code;

	# --- wrap the bugger in a do clause to keep shit local ---
	unless (defined $params->{inline}) {
		unshift @{$ptree}, [ ':', { code => 'do {' } ];
		push    @{$ptree}, [ ':', { code => '};'   } ];
	};

	return @{$ptree};
}

