#!/usr/bin/perl -w

# ===	change this stuff to suit your environment ============================

use lib '/web/cvs/puma';     # path to Puma (if not in Perl path)
#use lib '/usr/lib/perl5/site_perl';     # path to Puma (if not in Perl path)
my $etcdir = '/etc/puma';           # location of Puma's config.xml
my $configfile = 'config.xml';      # name of Puma's config.xml

# === end of local customizations ===========================================

use Puma::Util::Buffer;
use Puma::ServerPages::Server;
use Puma::Parser::ServerPages;
use Data::Dumper;
use Puma::ServerPages::Config 'getConfig';

use strict;

# --- declare variables in this scope ---
my ($server, $code, @errors);
my $filename = $ENV{PATH_TRANSLATED} || 'foo.psp';

# --- initialize output buffering class ---
my $buffer = new Puma::Util::Buffer;

# --- generate code from the .psp ---
my $startup = $buffer->capture(
	sub {
		# --- capture SIGWARN messages ---
		local $SIG{__WARN__} = sub { push @errors, $_[0] };

		# --- load the configuration data ---
		my $config = getConfig("$etcdir/$configfile");

		# --- create the server object (CGI) ---
		$server = getServer($config);

		# --- parse and codify ---
		my $parser = new Puma::Parser::ServerPages(Server => $server);
		my $ptree = $parser->parseFile($filename);
		$code = $server->getStartCode() . $parser->codify($ptree) . $server->getEndCode();
	}
);
# --- capture error messages ---
push @errors, $@ if $@;
push @errors, $startup if $startup;

# --- generate output from the code ---
my $output = $buffer->capture(
	sub {
		# --- capture SIGWARN messages ---
		local $SIG{__WARN__} = sub { push @errors, $_[0] };
		eval($code);
	}
);
# --- capture error messages ---
push @errors, $@ if $@;

# --- show the results ---
if (@errors) {

	# === ERRORS!! ===

	# --- stop any redirects before spitting out the header ---
	$server->redirect('');
	print $server->header;

	# --- show the errors ---
	print "<pre>Puma Server Pages Error : ";
	foreach my $err (@errors) { print "\n$err"; }
	print "</pre><hr>\n";

	# --- add line numbers to the code and display it ---
	my $lcode;
	my @lines = split /\n/, $code;
	my $linenumber = 1;
	foreach my $line (@lines) {
		$line =~ s/</&lt;/g;
		$lcode .= $linenumber++ .": $line\n";
	}
	print "<pre><code>$lcode</code></pre>\n";

} else {

	# === Clean Run ===

	# --- spit out the header ---
	print $server->header;
	# --- spit out the page unless there was a redirect ---
	print $output ."\n" unless $server->redirect;

	#$code =~ s/</&lt;/g; $code =~ s/>/&gt;/g;
	#$output =~ s/</&lt;/g; $output =~ s/>/&gt;/g;
	#print "<pre><hr>$output<hr>$code</pre>\n"
}

exit 1;

# === end of handler ========================================================

# --- method to instantiate the server object ---
sub getServer {
	my $config = shift;
	return undef unless defined $config;

	# --- get the server module name ---
	my $serverclass = $config->{module} || 'Puma::ServerPages::Server';
	return undef unless $serverclass;

	# --- instantiate the server object ---
	my $server;
	eval qq[
		use $serverclass;
		\$server = new $serverclass(Config => \$config);
	];

	return $server;
};
