package Puma::Socket::Listener;
use ibase Puma::Core;

use Socket;
use Symbol;
use Puma::Util::NCHash;
use Puma::Util::Conversion qw(phonepad);

use strict;

# === Constructor ===========================================================
sub construct {
	my ($self, $args) = (shift, nchash(@_));

	# --- get the rules for extracting valid labels ---
	$self->stripArgs($args, qw(config logfile module port));

	# --- convert the port number ---
	$self->{port} = phonepad($self->{port});

	# --- spawn the listener ---
	$self->{PID} = fork;
	# --- if we have a PID, this is the parent process ---
	if ($self->{PID} == 0) {
		# --- if the PID is 0, this is the child ---
		$self->listener;
		exit;
	} elsif (!defined($self->{PID})) {
		# --- if PID is undef, we have a problem ---
		logmsg("cannot fork: $!");
		return undef;
	}	

	return $self->SUPER::construct(%$args);
}

# === Listener ==============================================================
use IO::Handle qw(autoflush);
sub listener {
	my $self = shift;

	# --- define socket filehandles ---
	my $server = gensym();
	my $client = gensym();

	# --- prep the socket ---
	my $proto = getprotobyname('tcp');
	socket($server, PF_INET, SOCK_STREAM, $proto)
		or die "socket: $!";
	setsockopt($server, SOL_SOCKET, SO_REUSEADDR, pack("l", 1))
		or die "setsockopt: $!";
	bind($server, sockaddr_in($self->{port}, INADDR_ANY))
		or die "bind: $!";
	listen($server, SOMAXCONN)
		or die "listen: $!";

	logmsg("listener $self->{module} started on port $self->{port}");

	# --- listen for connections ---
	for (my $paddr; $paddr = accept($client, $server); close $client) {
		my ($port, $iaddr) = sockaddr_in($paddr);
		my $name = gethostbyaddr($iaddr, AF_INET) || '[unknown]';
		autoflush($client, 1);

		logmsg("connection from $name [", inet_ntoa($iaddr), "] at port $port");

		# --- load the server object ---
		my $serverobject;
		eval qq[
			use $self->{module};
			\$serverobject = new $self->{module}(
				Client => \$client,
				config => \$self->{config},
			);
		] || die "died $!\n";

		# --- spawn the listener ---
		my $pid = fork();
		# --- if we have a PID, this is the parent process ---
		if ($pid == 0) {
			# --- if the PID is 0, this is the child ---
			$serverobject->start();
			undef($serverobject);
			close $client;
			logmsg("disconnected");
			exit;
		} elsif (!defined($pid)) {
			# --- if PID is undef, we have a problem ---
			logmsg("cannot fork: $!");
			return 0;
		}
	}

	logmsg("listener $self->{module} killed on port $self->{port}");
}

# === Message Logger ========================================================
#! TODO: rewrite this in as a package in Puma::Util
sub logmsg { print "$0 $$: @_ at ", scalar localtime, "\n" }

1;
