eckage Puma::Socket::XML;
use ibase Puma::Socket::Server;

use strict;

# === Internal / Inherited API ==============================================

sub initialize {
	my $self = shift;
	print {$self->{Client}} "Connected to Puma::Socket::XML Server\n";
	print {$self->{Client}} "You can type, but it won't do you much good\n";
}

sub terminate  { print {$_[0]->{Client}} "Goodbye\n"; }

sub process {
	my $self = shift;
	my $socket = $self->{Client};

	# --- set the socket to be non-blocking ---
	use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
	my $flags;
	$flags = fcntl($socket, F_GETFL, 0)
		or die "Can't get flags for the socket: $!\n";
	$flags = fcntl($socket, F_SETFL, $flags | O_NONBLOCK)
		or die "Can't set flags for the socket: $!\n";

	$self->{data} = '';
	my ($timer, $done) = (0, 0);
	do {

		# --- check the stream for any data ---
		my $instr;
		my $count = read($socket, $instr, 1024);

		# --- if anything is read, add it to data ns reset the timer ---
		if ($count) {
			$self->{data} .= $instr;
			$timer = 0;
		}

		# --- sleep for 250 ms, increment the timer ---
		$timer += sleepms(250);

		# --- check for header ---
		if (!$self->{header} && $self->{data} =~ /(.*?)\r??\n\r??\n(.*)/s) {
			$self->{header} = $1;
			$self->{data} = $2;
		}

		# --- check for XML ---
		if ($self->{header} && $self->{data} =~ /<(\w\S*)/s) {
			$done = 1 if $self->{data} =~ /<\/$1/s;
		}

	} while ($timer < 15000 && !$done);

	if ($done) {
		use Puma::Util::XML 'xml2hash';

		$self->{xml} = $self->{data};
		$self->{xmlhash} = xml2hash(XML => $self->{xml});
		$self->processXML();
	}

	return 0;
}

# === This be the API! ======================================================
sub processXML {
	print "processing\n";
}

# === Utility Methods =======================================================
# --- a millisecond version of sleep ---
sub sleepms {
	my $ms = shift || 0;
	select(undef, undef, undef, $ms / 1000);
	return $ms;
}

1;
