package Puma::Tag;
use ibase Puma::Core;

=head1 NAME

Puma::Tag - an abstract base class for Puma tags

=head1 SYNOPSIS

 use ibase Puma::Tag;

=head1 DESCRIPTION

=head1 METHODS

=head2 render();

=head2 renderChildren();


=cut

sub render { }
sub renderChildren {
	my $self = shift;

	# --- get the current code ---
	return unless $self->{_codestack} && @{$self->{_codestack}};
	my $code = $self->{_codestack}->[0];

	return $code->();
}

sub _render ($&, $&) {
	my ($self, $method, $child) = @_;

	# --- push the code to the stack ---
	# TODO: find a more elegant way to write this
	#       hash-element-array-dereference nonsense
	my $cstack = $self->{_codestack} || [];
	unshift @$cstack, $child;
	$self->{_codestack} = $cstack;

	# --- call the public render routine ---
	my $retval = $method->(); #$self->render($name, %$params);

	# --- pop the code off the stack ---
	shift @$cstack;

	return $retval;
}

=head1 COPYRIGHT

Copyright (c) 2001, 2002 by Ingmar Ellenberger.

Distributed under The Artistic License.  For the text of this license,
see http://puma.site42.com/license.psp or read the file LICENSE in the
root of the distribution.

=cut

1;
