package Puma::Util::Buffer;

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=description

	Buffer.pm - An output capture and buffering class

=cut

use Symbol;

use strict;

sub new {
	# --- wierd constructor stuff for handle tie ---
	my $this = shift;
	my $handle = gensym;
	my $self = tie *{$handle}, ref($this) || $this;
	$self->{handle} = $handle;

	return $self;
}

sub startCapture {
	my $self = shift;
	$self->{oldfh} = select($self->{handle});
	return $self->handle;
}


sub endCapture {
	my $self = shift;
	if ($self->{oldfh}) {
		select $self->{oldfh};
	} else {
		select STDIN;
	}
	return $self->dump();
}

sub capture($&) {
	my ($self, $code) = @_;

	$self->startCapture;
	$code->();
	return $self->endCapture;
}

sub flush {
	my $self = shift;
	my $buffer = $self->{buffer};
	$self->{buffer} = '';
	return $buffer;
}

sub handle { my $self = shift; return $self->{handle}; }
sub dump { my $self = shift; return $self->{buffer}; }

# === handle tie stuff ======================================================

sub TIEHANDLE {
	my $this = shift;
	return bless {}, ref($this) || $this;
}

sub PRINT {
	my $self = shift;
	foreach my $arg (@_) { $self->{buffer} .= $arg if defined $arg; }
	return;
}

# TODO: implement methods below

sub WRITE {
	# $r = shift;
	# my($buf,$len,$offset) = @_;
	# print "WRITE called, \$buf=$buf, \$len=$len, \$offset=$offset";
}

sub PRINTF {
	# shift;
	# my $fmt = shift;
	# print sprintf($fmt, @_)."\n";
}

sub READ {
	# my $self = shift;
	# my $$bufref = \$_[0];
	# my(undef,$len,$offset) = @_;
	# print "READ called, \$buf=$bufref, \$len=$len, \$offset=$offset";
	# # add to $$bufref, set $len to number of characters read
	# $len;
}

sub READLINE {
	# $r = shift;
	# "READLINE called $$r times\n";
}

sub GETC {
	# print "Don't GETC, Get Perl";
	# return "a";
}

sub CLOSE {
	# print "CLOSE called.\n"
}


1;

__END__
