package Puma::Util::Conversion;
use ibase Exporter;

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=descripion

	A bunch of exported functions to do various conversions

=cut

@EXPORT = qw( phonepad nortelphonepad ctime2epoch );

use strict;

# === Phone Pad Conversions =================================================
# --- convert a string to a phone number ---
sub phonepad {
	my $num = shift;
	$num =~ s/[abc]/2/ig;
	$num =~ s/[def]/3/ig;
	$num =~ s/[ghi]/4/ig;
	$num =~ s/[jkl]/5/ig;
	$num =~ s/[mno]/6/ig;
	$num =~ s/[pqrs]/7/ig;
	$num =~ s/[tuv]/8/ig;
	$num =~ s/[wxyz]/9/ig;
	return $num;
}

# --- convert a string to a phone number, NorTel style ---
sub nortelphonepad {
	my $num = shift;
	$num =~ s/[abc]/2/ig;
	$num =~ s/[def]/3/ig;
	$num =~ s/[ghi]/4/ig;
	$num =~ s/[jkl]/5/ig;
	$num =~ s/[mno]/6/ig;
	$num =~ s/[prs]/7/ig;
	$num =~ s/[tuv]/8/ig;
	$num =~ s/[wxy]/9/ig;
	$num =~ s/[qz]/0/ig;
	return $num;
}

# === Date/Time Conversions =================================================

use Time::Local;
sub ctime2epoch {
	my $ctime = shift;

	# Mon Oct 22 09:26:21 2001
	my @ctarray = split /(?:\s|:)+/, $ctime;

	# --- convert day of week ---
	$ctarray[0] = 0 if $ctarray[0] =~ /^Sun/i;
	$ctarray[0] = 1 if $ctarray[0] =~ /^Mon/i;
	$ctarray[0] = 2 if $ctarray[0] =~ /^Tue/i;
	$ctarray[0] = 3 if $ctarray[0] =~ /^Wed/i;
	$ctarray[0] = 4 if $ctarray[0] =~ /^Thu/i;
	$ctarray[0] = 5 if $ctarray[0] =~ /^Fri/i;
	$ctarray[0] = 6 if $ctarray[0] =~ /^Sat/i;
	
	# --- convert month ---
	$ctarray[1] = 0  if $ctarray[1] =~ /^Jan/i;
	$ctarray[1] = 1  if $ctarray[1] =~ /^Feb/i;
	$ctarray[1] = 2  if $ctarray[1] =~ /^Mar/i;
	$ctarray[1] = 3  if $ctarray[1] =~ /^Apr/i;
	$ctarray[1] = 4  if $ctarray[1] =~ /^May/i;
	$ctarray[1] = 5  if $ctarray[1] =~ /^Jun/i;
	$ctarray[1] = 6  if $ctarray[1] =~ /^Jul/i;
	$ctarray[1] = 7  if $ctarray[1] =~ /^Aug/i;
	$ctarray[1] = 8  if $ctarray[1] =~ /^Sep/i;
	$ctarray[1] = 9  if $ctarray[1] =~ /^Oct/i;
	$ctarray[1] = 10 if $ctarray[1] =~ /^Nov/i;
	$ctarray[1] = 11 if $ctarray[1] =~ /^Dec/i;

	return timelocal(
		$ctarray[5], # seconds
		$ctarray[4], # minutes
		$ctarray[3], # hours
		$ctarray[2], # day of month
		$ctarray[1], # month
		$ctarray[6], # year
	);
}

1;
