package Puma::Util::NCHash;
use ibase Exporter;

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=descripion

	This is a wicked little class that makes hashed case-insensitive.

=cut

@EXPORT = qw( nchash );

use strict;

sub nchash { return new Puma::Util::NCHash(@_); }

# TODO: build config object to be passed in
#    $self->{_location} - db, path, whatever.
#    $self->{_context}  - application name
#    $self->{_uid}      - unique session key

# --- object contructor ---
sub new {
	# --- a hellofan effort to get a blessed obj ref to a tied hash! ---
	my $this = shift;                    # get classname or ref to instantiated object
	my $class = ref($this) || $this;     # get classname from ref if necessary
	my %hashtie = ();                    # create a hash to tie
	tie %hashtie, $class;                # tie hash to class (for hash tie)
	my $self = bless \%hashtie, $class;  # bless ref to tied hash into class (for object)

	$self->reserve( _content  => { @_ } );
	return $self;
}

# --- generate a regex of reserved words ---
sub reservedregex {
	my $self = shift;
	my $reserved =
		defined $self->{_reserved}
			? join('|', @{$self->{_reserved}})
			: '_reserved';
	return qr/$reserved/s;
}

sub reserve {
	my $self = shift;

	# --- make sure we have an even number of arguments ---
	my %args = @_ % 2 ? (@_, '') : @_;

	# --- create a new reserved list if it doesn't already exist ---
	$self->{_reserved} = [ '_reserved' ] unless defined $self->{_reserved};

	foreach my $key (keys %args) {
		# --- add the keys to the reserved list ---
		push @{$self->{_reserved}}, $key;
		# --- initiate the values for each key ---
		$self->{$key} = $args{$key};
	}
}

# === hash tie stuff ========================================================

# --- hash constructor ---
sub TIEHASH {
	my ($this, $hash) = (shift, shift || {});
	return bless $hash, ref($this) || $this;
}

sub STORE {
	my ($self, $key, $value) = @_;

	# --- set/return the value for reserved keys ---
	my $reserved = $self->reservedregex();
	return $self->{$key} = $value if $key =~ /$reserved/;

	# --- delete all old keys ---
	my $lckey = lc($key);
	foreach my $oldkey (keys %{$self->{_content}}) {
		delete $self->{_content}->{$oldkey} if lc($oldkey) eq $lckey;
	}
	# --- assign and return the new key ---
	return $self->{_content}->{$key} = $value;
}

sub FETCH    {
	my ($self, $key) = @_;

	# --- set/return the value for reserved keys ---
	my $reserved = $self->reservedregex();
	return $self->{$key} if $key =~ /$reserved/;

	# --- return the value for the first matching key ---
	my $lckey = lc($key);
	foreach my $oldkey (keys %{$self->{_content}}) {
		return $self->{_content}->{$oldkey} if lc($oldkey) eq $lckey;
	}
	return undef;
}

sub EXISTS   {
	my ($self, $key) = @_;

	my $lckey = lc($key);
	foreach my $oldkey (keys %{$self->{_content}}) {
		return 1 if lc($oldkey) eq $lckey;
	}
	return undef;
}

sub DELETE   {
	my ($self, $key) = @_;

	my $oldval;
	my $lckey = lc($key);
	foreach my $oldkey (keys %{$self->{_content}}) {
		$oldval = delete $self->{_content}->{$oldkey} if lc($oldkey) eq $lckey;
	}
	return $oldval;
}

sub FIRSTKEY {
	my $a = scalar keys %{$_[0]->{_content}};
	my $foo = each %{$_[0]->{_content}};
	return $foo;
}

sub NEXTKEY  {
	my ($self, $lastkey) = @_;

	foreach my $key (keys %{$self->{_content}}) {
		return $key unless $lastkey;
		$lastkey = '' if $key eq $lastkey;
	}
	return undef;
}

sub CLEAR    { %{$_[0]->{_content}} = () }1;

1;
