#!/usr/bin/perl -w

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=description

   The configuration script builds the installation file based on existing
   and new configuration parameters.

   It is much more preferable to place an XML configuration file in the
   search path rather than modify this file each time a new release of the
   package appears.

   Until proper documemtation is available, please read the code comments for
   a description of the available parameters

=cut

use lib '.';
use Puma::Util::XML qw( xml2hash hash2xml );

use strict;


# === Get base for 'config.xml' =============================================

# --- start with the 'config.base.xml' file ---
my $basefilename = './config.base.xml';
if (-e './config.xml') {
	# --- use 'config.xml from a previous run? ---
	print "\nI found a configuration that was previously generated\n";
	print "   Would you like to use this as your base configuration? y/[N] : ";

	my $prevconf = lc((readline (*STDIN) =~ /^\s*(.*?)\s*$/)[0] || 'N');
	$basefilename = './config.xml' if $prevconf eq 'y' || $prevconf eq 'yes';
}
# --- load up the base config file ---
my $config = xml2hash(FileName => $basefilename);


# === Determine the path of the 'etc' directory =============================

# --- grab directory name from existing config ----
my $etcdir = $config->{puma}->{install}->{etc}->{directory} || '/etc/puma';
$etcdir = '/usr/local/etc/puma' if -e '/usr/local/etc/puma/config.xml';

# --- ask the user ---
print "\nWhere do you want to store the config.xml for Puma?\n";
print "   [$etcdir] : ";

# --- clean-up and prep for outbound XML ---
$etcdir = (readline (*STDIN) =~ /^\s*(.*?)\s*$/)[0] || $etcdir;
$config->{puma}->{install}->{etc}->{directory} = $etcdir;

# --- read the 'config.xml' from $etcdir ---
$etcdir =~ s/\/*$//g;
my $etcconfig = {};
$etcconfig = xml2hash(FileName => "$etcdir/config.xml") if -e "$etcdir/config.xml";


# === Determine where to install the Perl modules ===========================

# --- grab directory name from existing config ----
my $perldir = $etcconfig->{puma}->{install}->{perl}->{directory} ||
              $config->{puma}->{install}->{perl}->{directory} ||
              '/usr/lib/perl5/site_perl';

# --- ask the user ---
print "\nWhere do you want to install the Puma perl modules?\n";
print "   [$perldir] : ";

# --- clean-up and prep for outbound XML ---
$perldir = (readline (*STDIN) =~ /^\s*(.*?)\s*$/)[0] || $perldir;
$config->{puma}->{install}->{perl}->{directory} = $perldir;

print "\n";


# === Build the new XML and write it out ====================================

# --- delete the install parameters from $etcconfig ---
delete $etcconfig->{puma}->{install};

# --- merge the hashes ---
print "merging configuration hashes ...\n";
$config = mergehash($config, $etcconfig);

# --- write the install XML ---
print "writing 'config.xml' ...\n";
hash2xml(
	XML => $config,
	FileName => 'config.xml',
);

print "done.\n\n";


# === Misc subs to make life easier =========================================

# --- overlay one hash (hash2) on another (hash1) ---
# TODO: move this code into Util::Config???
sub mergehash {
	my ($hash1, $hash2) = @_;

	foreach my $key (keys %{$hash2}) {
		if (exists $hash1->{$key} && ref($hash1->{$key}) eq 'HASH') {
			mergehash($hash1->{$key}, $hash2->{$key});
		} else {
			$hash1->{$key} = $hash2->{$key};
		}
	}

	return $hash1;
}
