#!/usr/bin/perl -w

=copyright

	Puma - Perl Universal Markup
	Copyright (c) 2001, Ingmar Ellenberger.  All rights reserved.
	Software is distributed AS IS.  No warranties, expressed or implied.

	This license is temporary, pending the selection of an appropriate
	open-source license.

=description

	Basic installation script.  Not much to describe for now.

=cut

use lib '.';
use Puma::Util::XML qw( xml2hash );

use strict;


# --- die if we can't find 'config.xml' ---
die "\nERROR: could not locate 'config.xml'\n\n" unless -e './config.xml';

# --- load installation configuration parameters ---
my $config = xml2hash(FileName => 'config.xml')->{puma}->{install} || {};
print "\n";

# --- install libraries ---
print "installing perl modules ...\n";
my $perldir = $config->{perl}->{directory} ||
	die "\nERROR: could not install perl modules\n".
	    "   no perl directory listed in 'config.xml'\n\n";
execute("rm -rf $perldir/Puma $perldir/ibase.pm") unless -d $perldir;
execute("mkdir -p $perldir") unless -d $perldir;
execute("cp -dfR Puma ibase.pm $perldir");

# --- install Puma etc's ---
print "installing Puma configuration files ...\n";
my $etcdir = $config->{etc}->{directory} ||
	die "\nERROR: could not install Puma configuration files\n".
	    "   no 'etc' directory listed in 'config.xml'\n\n";
execute("mkdir -p $etcdir") unless -d $etcdir;
execute("cp -f config.xml $etcdir");

print "done.\n\n";

# === Misc subs to make life easier =========================================

# --- execute shell command, die on error ---
sub execute {
	my $cmd = shift;
	my $error = `$cmd 2>&1`;
	$error && die "installation failed with the following error(s):\n\n$error\n";
}

